unit BindCompExprSampleFormUnit1;

interface

uses
  System.SysUtils, System.Types, System.Classes, System.Variants, FMX.Types, FMX.Controls, FMX.Forms,
  FMX.Dialogs, System.Rtti, FMX.Edit, Data.Bind.Components, FMX.Layouts, FMX.ListBox,
  Data.Bind.EngExt, FMX.Menus, Fmx.Bind.DBEngExt, FMX.Memo,
  System.Bindings.Outputs;

type
  TForm1 = class(TForm)
    BindingsList1: TBindingsList;
    BindEdits: TBindExpression;
    EditOutput: TEdit;
    EditSource: TEdit;
    LabelEditControl: TLabel;
    LabelEditSource: TLabel;
    ButtonEvaluate: TButton;
    CheckBoxActive: TCheckBox;
    BindScopeComponent1: TBindScope;
    ListBoxComponents: TListBox;
    Label1: TLabel;
    ListBox1: TListBox;
    LabelListBox: TLabel;
    LabelSourceExpressions: TLabel;
    LabelControlExpression: TLabel;
    EditControlExpression: TEdit;
    LabelControlComponent: TLabel;
    EditControlComponent: TEdit;
    LabelSourceComponent: TLabel;
    EditSourceComponentName: TEdit;
    LabelEditEnter: TLabel;
    LabelClickList: TLabel;
    MemoSourceExpression: TMemo;
    procedure ButtonEvaluateClick(Sender: TObject);
    procedure EditSourceExpressionChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EditSourceChange(Sender: TObject);
    procedure ListBox1Change(Sender: TObject);
    procedure BindEditsActivating(Sender: TObject);
    procedure CheckBoxActiveChange(Sender: TObject);
  private
    FChecking: Boolean;
    FLoaded: Boolean;
    procedure OnIdle(Sender: TObject; var Done: Boolean);
    procedure BindingNotify(AObject: TObject);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.fmx}

uses ExprSampleCommon, Fmx.Bind.Editors;

// Let the expression engine know about an change to an object so that
// managed expressions can be reevaluated
procedure TForm1.BindingNotify(AObject: TObject);
begin
  try
    BindingsList1.Notify(AObject, '');
  except
    on E: Exception do
      ShowNotifyError(AObject, E.Message);
  end;
end;

procedure TForm1.ButtonEvaluateClick(Sender: TObject);
begin
  try
    // Toggle active to use new expression (see BindEditsActivating)
    BindScopeComponent1.Active := False;
    BindScopeComponent1.Active := True;
  except
    on E: Exception do
    begin
      ShowExpressionError(BindEdits, E.Message);
    end;
  end
end;

procedure TForm1.BindEditsActivating(Sender: TObject);
begin
  if FLoaded then
    BindEdits.SourceExpression := MemoSourceExpression.Text;
end;


procedure TForm1.CheckBoxActiveChange(Sender: TObject);
begin
  if not FChecking then
    try
      BindScopeComponent1.Active := CheckBoxActive.IsChecked;
    except
      on E: Exception do
        ShowExpressionError(BindEdits, E.Message);
    end
end;

procedure TForm1.EditSourceExpressionChange(Sender: TObject);
begin
  if BindScopeComponent1.Active then
    ButtonEvaluateClick(Self);
end;


procedure TForm1.EditSourceChange(Sender: TObject);
begin
  // Notify so expressions referencing EditSource get reevaluated
  BindingNotify(Sender);
end;

procedure TForm1.ListBox1Change(Sender: TObject);
begin
  // Notify so expressions referencing ListBox1 get reevaluated
  BindingNotify(ListBox1);
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  I: Integer;
  LComponent: TComponent;
begin
  FLoaded := True;
  MemoSourceExpression.Text := BindEdits.SourceExpression;
  EditSourceComponentName.Text := BindEdits.SourceComponent.Name;
  EditControlComponent.Text := BindEdits.ControlComponent.Name;
  EditControlExpression.Text := BindEdits.ControlExpression;
  Application.OnIdle := OnIdle;
  // List component names, as information for use in writing expressions
  for I := 0 to Self.ComponentCount - 1 do
  begin
    LComponent := Self.Components[I];
    ListBoxComponents.AddObject(TListBoxItem.Create(ListBoxComponents));
    ListBoxComponents.Items[ListBoxComponents.Count-1] := LComponent.Name;
  end;

  // List sample values
  for I := 0 to 6 - 1 do
  begin
    ListBox1.AddObject(TListBoxItem.Create(ListBox1));
    ListBox1.Items[ListBox1.Count-1] := 'Item' + IntToStr(I);
  end;
  ListBox1.ItemIndex := 0;
end;

procedure TForm1.OnIdle(Sender: TObject; var Done: Boolean);
begin
  FChecking := True;
  try
    CheckBoxActive.IsChecked := BindScopeComponent1.Active;
    LabelEditEnter.Visible := BindScopeComponent1.Active;
    LabelClickList.Visible := BindScopeComponent1.Active;
  finally
    FChecking := False;
  end;

end;

end.
