unit BindDBExprSampleFormUnit1;

interface

uses
  System.SysUtils, System.Types, System.Classes, System.Variants, FMX.Types, FMX.Controls, FMX.Forms,
  FMX.Dialogs, System.Rtti, FMX.Edit,
  Data.Bind.Components, Data.Bind.DBScope, FMX.Layouts, FMX.ListBox, Data.DB,
  Datasnap.DBClient, Data.Bind.EngExt, FMX.Menus, Fmx.Bind.DBEngExt,
  Fmx.Bind.Editors, System.Bindings.Outputs;

type
  TForm1 = class(TForm)
    BindingsList1: TBindingsList;
    BindEdits: TBindExpression;
    EditOutput: TEdit;
    EditSourceExpression: TEdit;
    LabelEditOutput: TLabel;
    ButtonEvaluate: TButton;
    CheckBoxActive: TCheckBox;
    BindScopeDB1: TBindScopeDB;
    LabelSourceExpressions: TLabel;
    ButtonPrior: TButton;
    ButtonNext: TButton;
    ListBoxFieldNames: TListBox;
    Label3: TLabel;
    Label5: TLabel;
    ListBoxPersistentFields: TListBox;
    ClientDataSetDataSource1: TDataSource;
    ClientDataSet1: TClientDataSet;
    CategoryField: TStringField;
    SpeciesNameField: TStringField;
    LengthCmField: TFloatField;
    LengthInField: TFloatField;
    CommonNameField: TStringField;
    NotesField: TMemoField;
    GraphicField: TBlobField;
    GroupBox1: TGroupBox;
    LabelControlExpression: TLabel;
    EditControlExpression: TEdit;
    LabelControlComponent: TLabel;
    EditControlComponent: TEdit;
    EditSourceComponent: TEdit;
    LabelSourceComponent: TLabel;
    LabelClickNextPrior: TLabel;
    procedure ButtonEvaluateClick(Sender: TObject);
    procedure CheckBoxActiveClick(Sender: TObject);
    procedure EditSourceExpressionChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ButtonPriorClick(Sender: TObject);
    procedure ButtonNextClick(Sender: TObject);
    procedure ClientDataSetDataSource1DataChange(Sender: TObject;
      Field: TField);
    procedure BindEditsActivating(Sender: TObject);
  private
    FChecking: Boolean;
    FLoaded: Boolean;
    procedure OnIdle(Sender: TObject; var Done: Boolean);
  public
  end;

var
  Form1: TForm1;

implementation

uses ExprSampleCommon, System.Bindings.Helper;

{$R *.fmx}


procedure TForm1.ButtonEvaluateClick(Sender: TObject);
begin
  try
    // Toggle causes BindEditsActivating to get called
    BindEdits.Active := False;
    BindEdits.Active := True;
  except
    on E: Exception do
    begin
      ClientDataSetDataSource1.Enabled := False;
      ShowExpressionError(BindEdits, E.Message);
    end;
  end
end;

procedure TForm1.BindEditsActivating(Sender: TObject);
begin
  // Update expression before activate
  if FLoaded then
    BindEdits.SourceExpression := EditSourceExpression.Text;
end;

procedure TForm1.ButtonNextClick(Sender: TObject);
begin
  BindScopeDB1.DataSource.DataSet.Next;
end;

procedure TForm1.ButtonPriorClick(Sender: TObject);
begin
  BindScopeDB1.DataSource.DataSet.Prior;
end;

procedure TForm1.CheckBoxActiveClick(Sender: TObject);
begin
  if not FChecking then
  begin
    ClientDataSetDataSource1.Enabled := not ClientDataSetDataSource1.Enabled;
  end;
end;

procedure TForm1.ClientDataSetDataSource1DataChange(Sender: TObject;
  Field: TField);
var
  LField: TField;
begin
  // Call notify to cause expressions, which depend on fields, to be evaluated
  if Field <> nil then
    BindingsList1.Notify(Field, '')
  else
  begin
    Assert(Sender is TDataSource);
    for LField in (Sender as TDataSource).DataSet.Fields do
      BindingsList1.Notify(LField, '');
  end;

end;

procedure TForm1.EditSourceExpressionChange(Sender: TObject);
begin
  if BindEdits.Active then
    ButtonEvaluateClick(Self);
end;



procedure TForm1.FormCreate(Sender: TObject);
var
  LField: TField;
begin
  FLoaded := True;
  Application.OnIdle := OnIdle;
  ClientDataSetDataSource1.Enabled := False;
  EditSourceExpression.Text := Trim(BindEdits.SourceExpression);
  EditControlExpression.Text := BindEdits.ControlExpression;
  EditSourceComponent.Text := BindEdits.SourceComponent.Name;
  EditControlComponent.Text := BindEdits.ControlComponent.Name;
  EditOutput.Text := '';
  with ClientDataSetDataSource1.DataSet do
  begin

    // List field names as a convenience for typing in expressions which use fields
    for LField in Fields do
    begin
      ListBoxFieldNames.AddObject(TListBoxItem.Create(ListBoxFieldNames));
      ListBoxFieldNames.Items[ListBoxFieldNames.Count-1] := LField.FieldName;

      ListBoxPersistentFields.AddObject(TListBoxItem.Create(ListBoxPersistentFields));
      ListBoxPersistentFields.Items[ListBoxPersistentFields.Count-1] := LField.Name;
    end;
  end;
end;

procedure TForm1.OnIdle(Sender: TObject; var Done: Boolean);
begin
  FChecking := True;
  try
    CheckBoxActive.IsChecked := ClientDataSetDataSource1.Enabled;
    ButtonPrior.Enabled := not BindScopeDB1.DataSource.DataSet.BOF;
    ButtonNext.Enabled := not BindScopeDB1.DataSource.DataSet.EOF;
    LabelClickNextPrior.Visible := ClientDataSetDataSource1.Enabled;
  finally
    FChecking := False;
  end;
end;

end.
