unit BindExpressionSampleFormUnit1;

interface

uses
  System.SysUtils, System.Types, System.Classes, System.Variants, FMX.Types, FMX.Controls, FMX.Forms,
  FMX.Dialogs, System.Rtti, FMX.Edit, Data.Bind.Components, Data.Bind.EngExt, FMX.Menus,
  Fmx.Bind.DBEngExt, System.Bindings.Outputs;

type
  TForm1 = class(TForm)
    BindingsList1: TBindingsList;
    BindEdits: TBindExpression;
    EditOutput: TEdit;
    EditSource: TEdit;
    EditSourceExpression: TEdit;
    LabelEditOutput: TLabel;
    LabelEditSource: TLabel;
    ButtonEvaluate: TButton;
    CheckBoxActive: TCheckBox;
    EditSourceComponentName: TEdit;
    EditControlComponent: TEdit;
    EditControlExpression: TEdit;
    LabelPressEnter: TLabel;
    procedure ButtonEvaluateClick(Sender: TObject);
    procedure EditSourceExpressionChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EditSourceChange(Sender: TObject);
    procedure CheckBoxActiveChange(Sender: TObject);
    procedure BindEditsActivating(Sender: TObject);
  private
    FChecking: Boolean;
    FLoaded: Boolean;
    procedure OnIdle(Sender: TObject; var Done: Boolean);
    procedure BindingNotify(AObject: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.fmx}

uses ExprSampleCommon;

// Let the expression engine know about an change to an object so that
// managed expressions can be reevaluated
procedure TForm1.BindingNotify(AObject: TObject);
begin
  try
    BindingsList1.Notify(AObject, '');
  except
    on E: Exception do
      ShowNotifyError(AObject, E.Message);
  end;
end;

procedure TForm1.ButtonEvaluateClick(Sender: TObject);
begin
  try
    // Toggle causes BindEditsActivating to get called
    BindEdits.Active := False;
    BindEdits.Active := True;
  except
    on E: Exception do
    begin
      ShowExpressionError(BindEdits, E.Message);
    end;
  end
end;

procedure TForm1.BindEditsActivating(Sender: TObject);
begin
  // Update expression before evaluation
  if FLoaded then
    BindEdits.SourceExpression := EditSourceExpression.Text;
end;


procedure TForm1.CheckBoxActiveChange(Sender: TObject);
begin
  if not FChecking then
    try
      BindEdits.Active := CheckBoxActive.IsChecked;
    except
      on E: Exception do
        ShowExpressionError(BindEdits, E.Message);
    end
end;

procedure TForm1.EditSourceExpressionChange(Sender: TObject);
begin
  if BindEdits.Active then
    ButtonEvaluateClick(Self);
end;

procedure TForm1.EditSourceChange(Sender: TObject);
begin
  // Re-evaluate expressions which depend on edit text
  BindingNotify(Sender);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FLoaded := True;
  EditSourceExpression.Text := BindEdits.SourceExpression;
  EditSourceComponentName.Text := BindEdits.SourceComponent.Name;
  EditControlComponent.Text := BindEdits.ControlComponent.Name;
  EditControlExpression.Text := BindEdits.ControlExpression;
  Application.OnIdle := OnIdle;
end;

procedure TForm1.OnIdle(Sender: TObject; var Done: Boolean);
begin
  FChecking := True;
  try
    CheckBoxActive.IsChecked := BindEdits.Active;
    LabelPressEnter.Visible := BindEdits.Active;
  finally
    FChecking := False;
  end;
end;

end.
