unit ListExpressionsSampleFormUnit1;

interface

uses
  System.SysUtils, System.Types, System.Classes, System.Variants, FMX.Types, FMX.Controls, FMX.Forms,
  FMX.Dialogs, System.Rtti, FMX.Edit, Data.Bind.Components, Data.Bind.EngExt, FMX.Menus,
  Fmx.Bind.DBEngExt, System.Bindings.Outputs, FMX.ListBox, FMX.Layouts, SampleCollections,
  Generics.Collections,
  FMX.Bind.Editors; // Used by TBindList to populate a TListBox

type
  TForm1 = class(TForm)
    BindingsList1: TBindingsList;
    EditSourceExpression: TEdit;
    ButtonEvaluate: TButton;
    CheckBoxActive: TCheckBox;
    EditSourceComponentName: TEdit;
    EditControlComponent: TEdit;
    EditControlExpression: TEdit;
    BindList1: TBindList;
    ListBox1: TListBox;
    ComboBox1: TComboBox;
    BindScope1: TBindScope;
    CheckBoxAutoFill: TCheckBox;
    ButtonClear: TButton;
    Label1: TLabel;
    procedure ButtonEvaluateClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CheckBoxActiveChange(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
    procedure BindList1Activating(Sender: TObject);
    procedure ButtonClearClick(Sender: TObject);
    procedure CheckBoxAutoFillChange(Sender: TObject);
  private
    FChecking: Boolean;
    FDataObjects: TDictionary<TCollectionFactory, TObject>;
    FFactory: TCollectionFactory;
  public
    destructor Destroy; override;
    procedure OnIdle(Sender: TObject; var Done: Boolean);
    procedure UpdateDisplayFields;
  end;

var
  Form1: TForm1;

implementation

{$R *.fmx}

procedure TForm1.BindList1Activating(Sender: TObject);
begin
  if BindList1.FormatExpressions.Count > 0 then
    BindList1.FormatExpressions[0].SourceExpression := EditSourceExpression.Text;
end;

procedure TForm1.ButtonClearClick(Sender: TObject);
begin
  try
    BindList1.ClearList;
   except
    on E: Exception do
    begin
      ShowMessage(E.Message);
    end;
  end
end;

procedure TForm1.ButtonEvaluateClick(Sender: TObject);
begin
  try
    BindList1Activating(Self); // Update expression
    BindList1.FillList;
  except
    on E: Exception do
    begin
      ShowMessage(E.Message);
    end;
  end
end;

procedure TForm1.CheckBoxActiveChange(Sender: TObject);
begin
  if not FChecking then
    try
      BindList1.Active := CheckBoxActive.IsChecked;
    except
      on E: Exception do
        ShowMessage(E.Message);
    end
end;

procedure TForm1.CheckBoxAutoFillChange(Sender: TObject);
begin
  if not FChecking then
    try
      BindList1.AutoFill := CheckBoxAutoFill.IsChecked;
    except
      on E: Exception do
        ShowMessage(E.Message);
    end
end;

procedure TForm1.ComboBox1Change(Sender: TObject);
begin
  if ComboBox1.ItemIndex <> -1 then
  begin
    FFactory := ComboBox1.ListBox.Selected.Data as TCollectionFactory;
  end
  else
  begin
    FFactory := nil;
  end;
  if FFactory <> nil then
  begin
    BindScope1.DataObject := FFactory.CreateCollection;
    FDataObjects.AddOrSetValue(FFactory, BindScope1.DataObject);  // Track objects in order to free
    if BindList1.FormatExpressions.Count = 0 then
      BindList1.FormatExpressions.Add;
    BindList1.FormatExpressions[0].SourceExpression := FFactory.GetExpression;
    BindList1.FormatExpressions[0].ControlExpression := 'Text';
  end
  else
    BindScope1.DataObject := nil;
  UpdateDisplayFields;
end;

destructor TForm1.Destroy;
begin
  FDataObjects.Free;
  inherited;
end;

procedure TForm1.UpdateDisplayFields;
var
  LSourceExpression: string;
  LControlExpression: string;
  LSourceComponent: string;
  LControlComponent: string;
begin
  if BindList1.FormatExpressions.Count > 0 then
  begin
    LSourceExpression := BindList1.FormatExpressions[0].SourceExpression;
    LControlExpression := BindList1.FormatExpressions[0].ControlExpression;
  end;
  if BindList1.ControlComponent <> nil then
    LControlComponent := BindList1.ControlComponent.ClassName;
  if BindList1.SourceComponent <> nil then
  begin
    LSourceComponent := BindList1.SourceComponent.ClassName;
    if BindList1.SourceComponent is TBindScope then
      with TBindScope(BindList1.SourceComponent) do
        if DataObject <> nil then
          LSourceComponent := LSourceComponent + ' (' +
            DataObject.ClassName + ')'
      else if Component <> nil then
        LSourceComponent := LSourceComponent + ' (' +
          Component.ClassName + ')';
  end;
  EditSourceExpression.Text := LSourceExpression;
  EditControlExpression.Text := LControlExpression;
  EditControlComponent.Text := LControlComponent;
  EditSourceComponentName.Text := LSourceComponent;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  LFactory: TCollectionFactory;
  LListItem: TListBoxItem;
begin
  FDataObjects := TObjectDictionary<TCollectionFactory, TObject>.Create([doOwnsValues]);
  // List test data in combobox
  for LFactory in GetCollectionFactories do
  begin
    LListItem := TListBoxItem.Create(ComboBox1);
    ComboBox1.AddObject(LListItem);
    LListItem.Text := LFactory.DisplayName;
    LListItem.Data := LFactory;
  end;
  Application.OnIdle := OnIdle;
  UpdateDisplayFields;
end;

procedure TForm1.OnIdle(Sender: TObject; var Done: Boolean);
begin
  FChecking := True;
  try
    CheckBoxActive.IsChecked := BindList1.Active;
    CheckBoxAutoFill.IsChecked := BindList1.AutoFill;
  finally
    FChecking := False;
  end;
end;

end.
